# -*- coding: utf-8 -*-
from django.apps import AppConfig


class DJCMSCustomMenuConfig(AppConfig):
    name = 'djcms_custom_menu'
    verbose_name = 'Django CMS Custom Menu'

    def ready(self):
        import djcms_custom_menu.signals




import json
from pprint import pprint

from django.utils.functional import Promise
from django.utils.encoding import force_text

from django.contrib.sites.shortcuts import get_current_site
from django.conf import settings
from django.contrib import admin

from djcms_custom_menu.models import DJCMSCustomMenu
from djcms_custom_menu.nodes import get_nodes
from djcms_custom_menu.settings import ALLOWED_NAMESPACES

class LazyEncoder(json.JSONEncoder):
    """Encodes django's lazy i18n strings.
        Used to serialize translated strings to JSON, because
        simplejson chokes on it otherwise. """

    def default(self, obj):
        if isinstance(obj, Promise):
            return force_text(obj)
        return obj


class SimpleNode(object):
    id = None
    title = None
    url = None
    children = []

    def __init__(self, node):
        self.id = node.id
        self.title = node.title
        self.url = node.url


class DJCMSCustomMenuAdmin(admin.ModelAdmin):
    change_form_template = 'djcms_custom_menu/change_form.html'

    readonly_fields = ('site',)

    list_display = ('name', 'slug', 'site')

    def get_queryset(self, request):
        qs = super(DJCMSCustomMenuAdmin, self).get_queryset(request)
        current_site = get_current_site(request)
        return qs.filter(site=current_site)

    def add_view(self, request, form_url='', extra_context=None):
        if extra_context is None:
            extra_context = {}
        nodes, menu_renderer = get_nodes(request)
        available_pages = self.serialize_navigation(nodes)
        menu_pages = None
        extra_context = {
            'menu_pages': menu_pages,
            'available_pages': available_pages,
            'available_pages_json': json.dumps(available_pages, cls=LazyEncoder),
            'debug': settings.DEBUG,
        }
        return super(DJCMSCustomMenuAdmin, self).add_view(request, form_url, extra_context)

    def change_view(self, request, object_id, form_url='', extra_context=None):
        if extra_context is None:
            extra_context = {}
        nodes, menu_renderer = get_nodes(request)
        available_pages = self.serialize_navigation(nodes)
        menu_pages = DJCMSCustomMenu.objects.get(id=object_id).pages
        extra_context = {
            'menu_pages': menu_pages,
            'available_pages': available_pages,
            'available_pages_json': json.dumps(available_pages, cls=LazyEncoder),
            'debug': settings.DEBUG,
        }
        return super(DJCMSCustomMenuAdmin, self).change_view(request, object_id, form_url, extra_context)

    def serialize_navigation(self, all_nodes):
        # Recursively convert nodes to simple nodes
        cleaned = []
        for node in all_nodes:
            if not node.parent_id:
                cleaned_node = self.get_cleaned_node([node])
                if cleaned_node:
                    cleaned += cleaned_node

        return cleaned

    def get_cleaned_node(self, nodes):
        # Clean node to be a simple title/id/children class
        cleaned_nodes = []
        for node in nodes:
            # Limit the namespaces, typically to CMS Page only, can be set to None to ignore this
            if ALLOWED_NAMESPACES and node.namespace not in ALLOWED_NAMESPACES:
                continue
            cleaned_node = SimpleNode(node)
            if node.children:
                child_nodes = self.get_cleaned_node(node.children)
                if child_nodes:
                    cleaned_node.children = child_nodes
            cleaned_nodes.append(cleaned_node.__dict__)

        return cleaned_nodes # json.dumps(cleaned_nodes, cls=LazyEncoder)


admin.site.register(DJCMSCustomMenu, DJCMSCustomMenuAdmin)



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.sites.models import Site
from autoslug.fields import AutoSlugField
from jsonfield import JSONField
import collections


def get_current_site():
    return Site.objects.get_current().id


class DJCMSCustomMenu(models.Model):
    name = models.CharField(max_length=255)
    slug = AutoSlugField(always_update=False,
                         populate_from='name',
                         unique=True)
    pages = JSONField(blank=True,
                      null=True,
                      load_kwargs={
                          'object_hook': collections.OrderedDict
                      },
                      default=[])
    site = models.ForeignKey(Site,
                             on_delete=models.CASCADE,
                             help_text=_('The site the menu is accessible at.'),
                             verbose_name=_("site"),
                             default=get_current_site)

    def __str__(self):
        return self.name

    def __unicode__(self):
        return self.__str__()

    class Meta:
        verbose_name = "DJCMS Menu"
        verbose_name_plural = "DJCMS Menus"
        unique_together = ('slug', 'site',)



# -*- coding: utf-8 -*-
'''
Created on Jan 30, 2017

@author: jakob
'''
from django.conf import settings

CACHE_DURATION = getattr(settings, 'DJCMS_CUSTOM_MENU_CACHE_DURATION', 0)
ALLOWED_NAMESPACES = getattr(settings, 'DJCMS_CUSTOM_MENU_NAMESPACES', ['CMSMenu','DJCMSCustomMenu',])



# -*- coding: utf-8 -*-
'''
Created on May 21, 2016

@author: jakob
'''

from django.conf import settings
from django.core.cache import cache

from djcms_custom_menu.models import DJCMSCustomMenu
from djcms_custom_menu.settings import CACHE_DURATION


def flatten_menu(menu):
    ret = []
    for node in menu:
        ret.append(node)
        children = node.get('children', [])
        ret.extend(flatten_menu(children))
    return ret


def contains_page(menu, page_id):
    flat_menu = flatten_menu(menu)
    for node in flat_menu:
        if node['id'] == page_id:
            return True


def _key(menu_slug):
    return 'cms_named_menu_{slug}'.format(slug=menu_slug)


def get(menu_slug):
    key = _key(menu_slug)
    return cache.get(key, None)


def set(menu_slug, nodes):  # @ReservedAssignment
    key = _key(menu_slug)
    cache.set(key, nodes, CACHE_DURATION)


def delete(menu_slug):
    delete_many([menu_slug])


def delete_many(menu_slugs):
    for menu_slug in menu_slugs:
        key = _key(menu_slug)
        cache.delete(key)


def delete_by_page_id(page_id=None):
    menu_slugs = []
    # Will pick up any menu which already has the published page id - possibly.
    filter_string='"id":{}'.format(page_id)
    for menu in DJCMSCustomMenu.objects.filter(pages__contains=filter_string).all():
        if contains_page(menu.pages, page_id):
            menu_slugs.append(menu.slug)

    delete_many(menu_slugs)



# -*- coding: utf-8 -*-
from django.contrib.auth.models import AnonymousUser
from menus.menu_pool import menu_pool


def anonymous_request(f):
    def decorator(request, *args, **kwargs):
        auth_user = None
        # if request.user.is_authenticated():
        if request.user.is_authenticated:
            auth_user = request.user
            request.user = AnonymousUser()
        try:
            result = f(request, *args, **kwargs)
        finally:
            if auth_user is not None:
                request.user = auth_user
        return result

    return decorator


def filter_nodes(nodes):
    return [node for node in nodes if not node.attr.get('djcms_custom_menu_hidden', False) and node.attr.get("is_page", False)]


@anonymous_request
def get_nodes(request, namespace=None, root_id=None):

    # Set the menu renderer to force use of the anonymous version
    menu_renderer = menu_pool.get_renderer(request)

    nodes = menu_renderer.get_nodes(namespace, root_id, breadcrumb=False)

    nodes = filter_nodes(nodes)

    return nodes, menu_renderer




# /usr/bin/python3
import codecs
import os
from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

setup(
    name='djcms-custom-menu',
    version='1.0.1',
    description='An extension for Django CMS that allows you to create and edit multiple menus like Wordpress. This is compatible for Python 3.6, Django 2.1.9, django-cms 3.6.0',
    long_description=README,
    long_description_content_type='text/markdown',
    author='Abdullah Al Arafat Bipul',
    author_email='imbipul9@gmail.com',
    license=None,
    url='https://github.com/imbipul/djcms-custom-menu',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Django>=2.1.9',
        'django-classy-tags',
        'django-cms>=3.6.0',
        'jsonfield>=1.0.0',
        'django-autoslug>=1.7.2',
    ],
    package_data={
        'readme': ['README.rst'],
        'license': ['LICENSE']
    },
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Framework :: Django',
        'Framework :: Django :: 2.1',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



# -*- coding: utf-8 -*-
from djcms_custom_menu import cache
from djcms_custom_menu.models import DJCMSCustomMenu

from cms.models.titlemodels import Title
from django.db.models.signals import post_save, post_delete
from django.dispatch.dispatcher import receiver


@receiver(post_save, sender=DJCMSCustomMenu, dispatch_uid='clear_cache_djcms_custom_menu_saved')
def clear_cache_djcms_custom_menu_saved(sender, instance, **kwargs):
    cache.delete(instance.slug)


@receiver(post_delete, sender=DJCMSCustomMenu, dispatch_uid='clear_cache_djcms_custom_menu_deleted')
def clear_cache_djcms_custom_menu_deleted(sender, instance, **kwargs):
    cache.delete(instance.slug)


@receiver(post_save, sender=Title, dispatch_uid='clear_cache_title_saved')
def clear_cache_title_saved(sender, instance, **kwargs):
    cache.delete_by_page_id(instance.page.id)


@receiver(post_delete, sender=Title, dispatch_uid='clear_cache_title_deleted')
def clear_cache_title_deleted(sender, instance, **kwargs):
    cache.delete_by_page_id(instance.page.id)
